;-----------------------------------------------------------------------------
;NAME
;   add-log.ml -- maintain a change log for the current directory
;
;SYNOPSIS
;   add-change-log-entry	add an entry to the log
;   insert-log			add an entry to the current file at dot
;
;KEY BINDINGS
;   None.
;
;SEE ALSO
;   whist.ml
; 
;DESCRIPTION
;   This function helps you maintain a file called ChangeLog describing
;(presumably) changes to a code system. When called, it goes to the beginning
;of the file and inserts an entry header containing your user name, system
;name, and the time and date. It then sets margins and accepts text entry.
;
;HISTORY
; Wed Aug 29 15:25:16 1984  John McLaughlin  (mcl at unipress)
;   Added insert-log so that this can be done any where.
; 
;   3-July-84  Eric Raymond at Rabbit Software
;	Changed to use rebind and added this documentation header.
;   
;-----------------------------------------------------------------------------

(if (error-occurred (path-of "finger.ml"))
   (error-message (concat "whist.ml needs a file called finger.ml"
                          " and it does not exist"))
   (autoload "finger-name" "finger.ml")
)

(defun
    (add-change-log-entry
	(visit-file "ChangeLog")
	(setq mode-string "log")
	(local-rebind "justify-paragraph" "indent-C-procedure" "\ej")
	(beginning-of-file)
	(insert-log)
	(newline)
	(setq left-margin 9)
	(setq right-margin 75)
	(to-col left-margin)
	(newline-and-backup)
	(newline-and-backup)
    )
)

(defun 
  (insert-log time-string
          (setq time-string (current-time))
          (insert-string (substr time-string 9 2)  "-"
                         (substr time-string 5 3)  "-"
                         (substr time-string -2 2) "  " (finger-name)
                         " (" (users-login-name) ") at " (system-name)
	  )
  )
)

; add-log.ml ends here -------------------------------------------------------
