; cmucs-misc.ml
;
; miscellaneous functions for improving unix emacs
;
; backward-column		moves the cursor backward one column.
;				if the prev char is a tab, spaces are
;				added to make up for it.
;
; display-line-number		like (line-number) but also includes
;				the value of dot & buffer-size.
;
; write-file-push		writes all modified files and calls
;				up another monitor.
;
; write-file-pause		writes all modified files and pauses
;				back to the shell.  This is just like
;				(pop-out-of-emacs).
;
; dec-shell			calls up a shell but rebinds the
;				interrupt and erase characters to
;				^C and DELETE, respectively.
;
; control-c			sort of like exit-emacs, but
;				it prompts the user to make sure
;				he really wants to exit (regardless
;				of modified files).
;
; beginning-of-next-line	Pretty much what it looks like.
;
; set-bounds			when given a prefix-argument, it
;				narrows the region.  The same command
;				un-narrows it.
;
; insert-date			inserts the current date as a string.
;				If a prefix-argument is given, it strips
;				out the time string.
;
; match-begin			not sure
;
;
(autoload "copy-lines"			"killring.ml")
(autoload "backward-kill-character"	"killring.ml")
(autoload "send-int-signal"		"process.ml")
(autoload "copy-lines"			"killring.ml")
(autoload "unkill"			"killring.ml")

(defun
    (backward-column col
	(if (= (preceding-char) '^I')
	    (progn
		(setq col (current-column))
		(delete-previous-character)
		(while (< (current-column) col)
		       (insert-character ' ')
		)
	    )
	)
	(backward-kill-character)
    )
    (display-line-number orig page line pagetop
	(save-excursion
	    (setq orig (dot))
	    (beginning-of-file)
	    (setq page 1)
	    (setq pagetop (dot))
	    (while
		(& (! (error-occurred (search-forward "\^L")))
		   (<= (dot) orig))
		(++ page)
;		(setq page (+ page 1))
		(setq pagetop (dot))
	    )
	    (goto-character pagetop)
	    (setq line 1)
	    (while
		(& (! (error-occurred (search-forward "\n")))
		   (<= (dot) orig))
;		(setq line (+ line 1))
		(++ line)
	    )
	)
	(message "Line " line 
	    (if (> page 1)
		(concat " in page " page)
		""
	    )
	    ", " (- (dot) 1) " of " (buffer-size) " characters"
	)
    )
    (write-file-push
	(error-occurred (write-modified-files))
	(return-to-monitor)
    )
    (write-file-pause
	(error-occurred (write-modified-files))
	(pause-emacs)
    )
    (dec-shell
	(error-occurred (write-modified-files))
	(shell)
	(setq right-margin 999)
	(local-bind-to-key "backward-kill-character" "@")
	(local-bind-to-key "send-int-signal" ""))
    (control-c
        (if (| (> (recursion-depth) 0)
	       (= (substr (get-tty-string "Do you really want to exit? ")
			  1 1)
		  "y")
	    )
	    (exit-emacs)
	)
	(novalue)
    )
    (beginning-of-next-line
	(if (error-occurred 
		(prefix-argument-loop (search-forward "\n")))
	    (end-of-file)))
    (set-bounds
	(if prefix-argument-provided
	    (narrow-region)
	    (widen-region)))
    (insert-date
	(set-mark)
	(insert-string (current-time))
	(if prefix-argument-provided
	    (progn
		(backward-word)
		(exchange-dot-and-mark)
		(provide-prefix-argument 3 (forward-word))
		(forward-character)
		(erase-region)
		(forward-word))))
    (match-begin
	(save-excursion
	    (end-of-line)
	    (copy-lines)
	    (unkill)
	    (search-reverse "begin")
	    (delete-next-word)
	    (insert-string "end")
	)
    )
)
