; %Z%%M%  vers. %I%  last mod %G% %U%  %Q%
;---------------------------------------------------------------------------
;NAME
; 
; find-file.ml
; 
; Copyright (c) 1984 - Community Data Processing
;
;SYNOPSIS
; 
; (find-file)			- Does ITS style find-file
; (toggle-buffers)		- Does ITS style buffer movement
; (insert-buffer)		- obvious
; (revert-file)			- Does ITS style revert
; 
;KEYBINDINGS
;
; find-file	   ^X^F
; toggle-buffers   ^Xb
; 
;DESCRIPTION
; 
; Give the user help in finding a file:
;  buffer has associated file: present same path, and put file in other window
;  buffer has no file: present working directory, and put file in same window
; if you don't like the path you're presented with, use ^W to erase
;  components, or ^A^K to kill the whole thing
; ESC or SP can be used for each component of a path
; 
; Toggle buffers will switch to an already existing buffer.  If no buffer
; is specified then the default buffer is switched to.  The defualt buffer
; is the last visited buffer.
; Find file updates the global variable so toggle-buffers works correctly.
;
;HISTORY
;
; Tue Mar 12 12:30:54 1985  Harold Ancell (hga at teddy-bear)
; 	incorporated tog-buffs.ml which was created by
; 	John McLaughlin  (mcl at unipress) on Thu Jul 26 15:42:51 1984
;
; sw: 7/23/84
;	created
;----------------------------------------------------------------------


(declare-global &last-visited-buffer)
(temp-use-buffer "")

(defun (toggle-buffers &new-buff
  (if (= &last-visited-buffer "") 
     (setq &last-visited-buffer (current-buffer-name)))
  (setq &new-buff (get-tty-string
                     (concat "Buffer (default "
			     (concat &last-visited-buffer "): "))))
  (if (= &new-buff "") (setq &new-buff &last-visited-buffer))
  (setq &last-visited-buffer (current-buffer-name))
  (switch-to-buffer &new-buff))
)

(defun
  (buffer-file-path path file-name	;return path for this buffers file
    (setq path (working-directory))	;in case no file
    (if (!= (substr path -1 1) "/")	;no final slash?
	(setq path (concat path "/")))	;then add it
    (setq file-name (current-file-name));get file name of buffer
    (if (> (length file-name) 0)	;buffer has a file?
	(save-excursion			;save cuz of temp-use-buffer
	    (temp-use-buffer "path-buffer") ;get a work area
	    (setq needs-checkpointing 0);no need for checkpointing
	    (setq buffer-is-modified 0)	;nor saving
	    (erase-buffer)		;clear it
	    (set-mark)			;set mark at front
	    (insert-string file-name)	;stuff in file name
	    (search-reverse "/")	;go back to last slash
	    (forward-character)		;move over it
	    (setq path (region-to-string "path-buffer")))) ;get path
    path)				;return path
    
(find-file path &new-file
    (if (interactive)
	(setq path (buffer-file-path)))	;get path of buffers file
    (if (& (< (window-height) split-height-threshhold) ;not split window..
	   (= (length (current-file-name)) 0)) ;...& in buffer w/out file?
	(next-window))			;then switch so switch back
    (if (interactive)			;user called from keyboard?
	(progn				;then help her out
	    (push-back-string path)	;type the path for her
	    (setq &new-file
		  (get-tty-file ": find-file (use SPACE, ^W, ^A^K): "))
	    (setq &last-visited-buffer (current-buffer-name))
	    (visit-file &new-file))
	(progn
	      (setq &last-visited-buffer (current-buffer-name))
	      (visit-file (arg 1))))
    (novalue))
    
(insert-buffer buffer			;insert passed buffer at point
    (setq buffer (current-buffer-name))	;remember where we start
    (save-restriction			;save callers region
	(set-mark) (narrow-region)	;narrow down to nothing
	(save-excursion			;we are leaving...
	    (switch-to-buffer
		(get-tty-buffer "insert-buffer (use SPACE, ?): "))
	    (mark-whole-buffer)		;want whole buffer
	    (copy-region-to-buffer buffer)))) ;copy to our original buffer
    
(revert-file cursor			;copy file into buffer
    (setq cursor (+ 0 (dot)))		;want number not marker, because...
    (read-file (current-file-name))	;reread file (wiping out markers)
    (goto-character cursor))		;try to return to same place
    
)					;end of defuns
