;-----------------------------------------------------------------------------
; fortune.ml -- useful functions for setting up a Fortune terminal
;
;SYNOPSIS
;   function-key	-- returns key-sequence value of nth function key
;
;KEY BINDINGS
;   See below.
;
;DESCRIPTION
;   This package, when loaded, sets up nice global bindings for the special
; keys on a Fortune Systems terminal. It also provides an easy way of
; generating the values of the remaining (numbered) function keys for use
; in the remainder of the user's profile.
;
;NOTE
;   In case your site lacks one, here's the Fortune termcap description we
; use:
;
;FT|fos|fortune|Fortune 32-16 asynch:\
;	:is=^_..:\
;	:li#25:co#80:\
;	:am:bs:bw:\
;	:cl=20^L:cd=3*^\Y:\
;	:ic=5^\Q:dc=5^\W:\
;	:dl=15^\R:al=15^\E:\
;	:cm=^\C%+ %+ :\
;	:ta=^Z:\
;	:ho=10^^:do=3^J:up=3^K:bs=^H:\
;	:kb=^H:kd=^Ay\r:kh=^A?\r:kl=^Aw\r:kr=^Az\r:ku=^Ax\r:\
;	:nl=5^J:\
;	:so=^\H`:se=^\I`:sg=0:\
;	:us=^\HP:ue=^\IP:ug=0:\
;	:ce=^\Z:
;
;HISTORY
;   04-May-84  Eric S. Raymond at Rabbit Software
;	Created.
;-----------------------------------------------------------------------------

; Named motion and editing keys
(bind-to-key "delete-previous-character"	"\^h")		; BACKSPACE
(bind-to-key "info"				"\^A@\r")	; HELP
(bind-to-key "execute-extended-command"		"\^Aq\r")	; EXECUTE
(bind-to-key "newline-and-backup"		"\^Ar\r")	; INSERT
(bind-to-key "previous-page"			"\^As\r")	; PREV SCRN
(bind-to-key "delete-next-character"		"\^At\r")	; DELETE
(bind-to-key "next-page"			"\^Au\r")	; NEXT SCRN
(bind-to-key "backward-character"		"\^Aw\r")	; <
(bind-to-key "previous-line"			"\^Ax\r")	; ^
(bind-to-key "next-line"			"\^Ay\r")	; v
(bind-to-key "forward-character"		"\^Az\r")	; >

;-----------------------------------------------------------------------------
; Give the following functions a single numeric argument each.
; They will return the keystroke value of the corresponding numbered key.
;
; Use these with (bind-to-key) and (local-bind-to-key) as in
;    (bind-to-key "next-paragraph" (function-key 10))
;
; Note the following equivalences:
;   F0  = HELP      F17 = EXECUTE,    F18 = INSERT,      F19 = PREV SCRN,
;   F20 = DELETE,   F21 = NEXT SCRN,  F22 = RETURN,      F23 = LEFT ARROW,
;   F24 = UP ARROW, F25 = DOWN ARROW, F26 = RIGHT ARROW
; There are distinct shifted versions of all of these.
; The F24 return key is the one to right of the numeric keypad.
;-----------------------------------------------------------------------------

(defun
	(function-key
		(concat "\^A" (char-to-string (+ (arg 1) 96)) "\r")
	)
	(shifted-function-key
		(concat "\^A" (char-to-string (+ (arg 1) 64)) "\r")
	)
)

; fortune.ml ends here -------------------------------------------------------
