; ftp-visit.ml
;
;  UNIX Emacs package to simulate a "visit" on remote files
;  Copyright 1982 by Nathaniel Borenstein
;
;  CMU is pretty hardcoded in this one...
; 
;  The only function here you want to use is ftp-visit.  It will visit
;    a file on a remote machine, more or less.  It does not save the
;    file automatically, but merely puts it in a buffer with no
;    filename associated.  Of course, you can save it with ^X^W.
;    It uses the same buffer each time, so that each use trashes the
;    previously visited remote file.
; 
;  It tries to be smart about what machine to go to.  What this
;    means is that if the file name starts with '/usr' and the next
;    letter isn't another '/', then it uses that letter as the machine
;    name.  Otherwise it must be supplied, either interactively or as
;    the second argument.  A simple-minded kind of parsing is done to
;    select the "default" machine in such cases.
; 
;  Great care is taken to properly quote file names through to cmuftp. 
;    It is the author's belief that you need never quote anything in
;    the file name you pass or give to ftp-visit.  Just type the file
;    name as you'd give it on the target machine.
;
;    If the variable ignore-first-ftp-visit-output is set, we do just
;    that, thus accomodating an oddity of the c shell.
; 
(if (! (is-bound ignore-first-ftp-visit-output))
    (setq-default ignore-first-ftp-visit-output 0)
)
(declare-global .has-printed .machine .remotefile first-output) 
(defun 
    (ftp-visit .cmdname .default .prompt 
	(progn 
	       (save-window-excursion (list-processes))
	       (setq first-output "")
	       (setq .has-printed 0)
	       (setq .remotefile (concat "\"" (arg 1 "Remote file name? ") "\""))
	       (if (& (< (nargs) 2) 
		      (= "\"/usr" (substr .remotefile 1 5))
		      (!= "\"/usr/" (substr .remotefile 1 6))
		   )
		   (setq .machine (substr .remotefile 6 1))
		   (progn 
			(setq .default "g")
			(if (find-char-in-string "<" .remotefile)
			    (setq .default "cmuc")
			    (if (find-char-in-string "[" .remotefile)
				(setq .default "cmua")
			    )
			)
			(if (>= (nargs) 2)
			   (setq .machine 
			      (arg 2 (concat "Machine name [" .default "] ? "))
			   )
			   (setq .machine 
			      (get-tty-string (concat "Machine name [" .default "] ? "))
			   )
			)
			(if (= .machine "")
			    (setq .machine .default)
			)
		   )
	       )
	       (if (= .machine "a") (setq .machine "cmua"))
	       (setq .cmdname  (concat "cmuftp rq " .machine " -=" .remotefile ""))
	       (if (= (getenv "SHELL") "/bin/csh")
		   (setq .cmdname (concat "set prompt = \"\"; " .cmdname))
	       )
	       (error-occurred (delete-buffer "ftp.output"))
	       (if (= (getenv "SHELL") "/bin/csh")
		   (start-filtered-process .cmdname "cmuftp" ".dump-ftp-output-csh")
		   (start-filtered-process .cmdname "cmuftp" ".dump-ftp-output-sh")
	       )
	       (save-window-excursion 
		   (progn (pop-to-buffer "cmuftp")
			  (setq needs-checkpointing 0)
		   )
	       )
	       (pop-to-buffer "ftp.output")
	       (setq needs-checkpointing 0)
	       (setq mode-string "Waiting...")
	       (novalue)
	)
    )
    (.dump-ftp-output-csh
     (save-window-excursion 
	 (pop-to-buffer "ftp.output")
	 (end-of-file)
         (setq mode-line-format 
	     (concat "%[%M%] " .remotefile " on " .machine " "))
	 (if (| .has-printed (! ignore-first-ftp-visit-output))
	     (insert-string (process-output))
	 )
	 (setq .has-printed 1)
     )
    )
    (.dump-ftp-output-sh
     (save-window-excursion 
	 (pop-to-buffer "ftp.output")
	 (end-of-file)
	 (insert-string (process-output))
	 (if (!= 1 (process-status "cmuftp"))
	     (setq mode-line-format (concat "%[%M%] " .remotefile " on " .machine " "))
	 )
     )
    )
    (find-char-in-string i foundit
	(progn 
	    (setq foundit 0)
	    (setq i (length (arg 2)))
	    (while (>= i 1)
		(progn 
		   (if (= (arg 1) (substr (arg 2) i 1))
		       (setq foundit i)
		   )
		   (setq i (- i 1))
		)
	    )
	    foundit
	)
    )
)
(novalue)
