;------------------------------------------------------------------------------
;NAME
;   occur.ml -- show all occurrances of a pattern in a buffer
;
;SYNOPSIS
;   Occurrances	-- display all lines containing a given RE
;
;KEY BINDINGS
;   None.
;
;DESCRIPTION
;   The occur package allows one  to  find  the occurances of a string in a
;buffer.  It contains one function
;
;Occurrances    When invoked, prompts with "Search for all occurances of: "
;               and  then lists  (in a new buffer) all lines containing the
;               regular expression you type following dot. Possible options
;		(listed at the  bottom  of  the  screen)  allow you to page
;		through the listing buffer or abort the function.
;
;In addition, a global variable controls the action of the function:
;
;&Occurrances-Extra-Lines
;               is  a  global  variable  that  controls  how   many   extra
;               surrounding  lines  are  printed  in  addition  to the line
;               containing the string found.  If this variable is 0 then NO
;               additional lines are printed.  If this variable is  greater
;               than  0  then it will print that many lines above and below
;               the line on which the string was found.  When printing more
;               than one line per match in this fashion, it will also print
;               a seperator of '----------------' so you can tell where the
;               different matches begin and end.  At the end of the  buffer
;               it prints '<<<End of Occur>>>'.
;
;HISTORY
; [1] Aug 15 1984  McL: Fixed a couple more bugs.  The modeline used to give
; 			the file name as the position and the seperator of
;	 		dashes needed a newline after it.
;   04-Jul-84  Eric S. Raymond at Rabbit Software
;	Fixed the misspelling 'Occurance', made standard documentation header.
;	Changed code to use regular-expression search.
;   09-Apr-81 (06:36:37)  Doug Philips
;	Added the &Occurrances-Extra-Lines variable.
;   02-Feb-81 (06:11:03)  Doug Philips
;	Occurrances implemented, to imitate the $XOccurrances command in
;	Twenex Emacs. Unfortunately, this Emacs doesn't allow one to scribble
;	on the screen, and then have emacs restore the screen, which would be
;	the way to go with this command.
;------------------------------------------------------------------------------

(declare-global &Occurrances-Extra-Lines)
(if (= "" &Occurrances-Extra-Lines)
   (setq &Occurrances-Extra-Lines 0)
)

(defun
   (Occurrances occ-string c buf-name bpp o-count l-count temp-pos Occur-String
      (setq o-count (setq l-count 0))
      (setq buf-name (current-buffer-name))
      (setq bpp (dot))
      (if (arg 1)
         (setq occ-string (arg 1))
         (setq occ-string (get-tty-string "Search for all occurrances of: "))
      )
      (switch-to-buffer "excursions")
      (erase-buffer)
      (switch-to-buffer buf-name)
      (goto-character bpp)
      (while (! (error-occurred (re-search-forward occ-string)))
	 (setq o-count (+ 1 o-count))
         (setq l-count &Occurrances-Extra-Lines)
	 (beginning-of-line)(setq temp-pos (dot))
	 (while (> l-count 0)
            (&PL-Begin)
            (setq l-count (- l-count 1))
         )
	 (set-mark)
         (goto-character temp-pos)
         (&NL-Begin)
	 (setq temp-pos (dot))
         (setq l-count &Occurrances-Extra-Lines)
	 (while (> l-count 0)
            (&NL-Begin)
            (setq l-count (- l-count 1))
         )
	 (setq Occur-String (region-to-string))
	 (switch-to-buffer "excursions") (end-of-file)
	 (if (< 0 &Occurrances-Extra-Lines)
	    (insert-string "----------------\n")         ;  {1} 
         )
	 (insert-string Occur-String)                  
	 (switch-to-buffer buf-name)
         (goto-character temp-pos)
      )	 ;;; End of while loop searching
   (switch-to-buffer "excursions")
   (save-excursion (end-of-file)(insert-string "<<<End of Occur>>>\n"))
   (setq mode-line-format (concat "Found " o-count 
                                  " occurrances of '" occ-string
                                  "' after position: " bpp " in: " buf-name)
   )
   (beginning-of-file)
   (while (! (eobp))
      (message "--More?-- (sp=next page, ^G=abort, anything else is executed)")
      (setq c (get-tty-character))	 ; grab a character
      (if (= c ' ')
	 (progn
            (end-of-window) 	 ; keep going
            (if (! (eobp))	 ; but not if on last page!
	       (next-page)
            )
         )
	  (= c 7)
         (end-of-file)	 	 ; quit
	 (progn
            (push-back-character c); execute-it
	    (end-of-file)
         )
      )
   )
   (switch-to-buffer buf-name)
   (goto-character bpp)
   o-count
   )  ;;; End of Occurrances

   (&NL-Begin
      (next-line)
      (beginning-of-line)
   )

   (&PL-Begin
      (previous-line)
      (beginning-of-line)
   )
)

; occur.ml ends here ----------------------------------------------------------
