; New bindings:
;       ), }    parenthesis matcher (from Doug Phillips)
;       ESC-p   make a procedure or function (from Doug Phillips)
;       ^M      indent pascal (try it out)
;       ^X->    Indent this line by two.
;       ^X-<    Dedent this line by two.
;       current-indent (default 2) determines size of indenting
; all pascal keywords are abbreved and capitalized (by David Medinets)

(declare-global &tem &var)
(declare-global comment_mode)
(declare-global &comment-switch)
(declare-global &switch-to-arg)
(declare-global &comment-starter)
(declare-global &comment-marker)
(declare-global &argument-marker)
(declare-global pas_indent)
(declare-global &arg-name)
(declare-global &arg-bool)
(declare-global &dummy)
(declare-global &if-else-dummy)
(declare-global current-indent)
(declare-global indent-change)

(defun
   (pascal-mode
	(setq &arg-bool "0")
	(setq comment_mode "1")
	(setq &comment-switch "50")
	(setq &comment-starter "")
	(setq &switch-to-arg "1")
	(setq pas_indent "1")
	(setq &if-else-dummy "0")
        (setq-default indent-change 2)
        (setq mode-string "pascal")
        (use-syntax-table "pascal")
	(quietly-read-abbrev-file "pascal.abbrevs")
	(use-abbrev-table "pascal")
        (setq abbrev-mode 1)	
        (local-bind-to-key "indent-pascal" "
")
        (local-bind-to-key "newline" "
")
        (local-bind-to-key "make-pascal-structure" "p")
        (local-bind-to-key "show-matching-paren" ")")
        (local-bind-to-key "leave-and-show" "}")
        (local-bind-to-key "indent-region" ">")
        (local-bind-to-key "dedent-region" "<")
        (local-bind-to-key "enter-comment-mode" "{")
        (local-bind-to-key "toggle-comment-mode" "'")
	(modify-syntax-entry "w    _")
	(modify-syntax-entry "(){ *(")
	(modify-syntax-entry "   })*")
	(modify-syntax-entry ")(   )")
	(modify-syntax-entry "(]   [")
	(modify-syntax-entry ")[   ]")
	(modify-syntax-entry "(}{  {")
	(modify-syntax-entry "){ } }")
	(define-hooked-local-abbrev "begin" "BEGIN"  "begin-end-pair")
	(define-hooked-local-abbrev "repea" "REPEAT" "repeat-until-pair")
	(define-hooked-local-abbrev "while" "WHILE"  "while-do-pair")
	(define-hooked-local-abbrev "for"   "FOR"    "for-do-pair")
	(define-hooked-local-abbrev "if"    "IF"     "if-then-pair")
        (error-occurred (pascal-mode-hook))
        (novalue)))
    
(defun 
    (cond n running
          (setq n 1)
          (setq running 1)
          (while (& running (< n (nargs)))
                 (if (arg n)
                     (setq running 0)
                     (setq n (+ n 2))))
          (arg (+ n 1))))

(defun 
    (toggle-comment-mode
        (if (= abbrev-mode 0)
            (setq abbrev-mode 1)
            (setq abbrev-mode 0))
        (insert-character (last-key-struck))))

(defun 
        (enter-comment-mode
                (setq abbrev-mode 0)
                (insert-character (last-key-struck))))
    
(defun 
    (leave-comment-mode
        (setq abbrev-mode 1)
        (insert-character (last-key-struck))))
    
; This function gets bound to ')' -- it flashes to the corresponding
; '(' and fixes the indentation of the ')'.
(defun 
   (show-matching-paren dot instabs
	(if (eolp) (delete-white-space))
	(setq instabs (bolp))
	(setq dot (dot))
	(insert-character (last-key-struck)); instead of right paren
	(save-excursion
	    (backward-paren)
	    (if instabs (save-excursion descol
			    (setq descol (current-column))
			    (goto-character dot)
			    (to-col descol)))
	    (if (dot-is-visible)
		(sit-for 5)
		(progn
		      (beginning-of-line)
		      (set-mark)
		      (end-of-line)
		      (message (region-to-string)))))))

(defun 
    (leave-and-show
        (setq abbrev-mode 1)
        (show-matching-paren)))
    
(defun 
    (new-leave-and-show
        (setq abbrev-mode 1)
        (if (error-occurred (show-matching-paren))
            (error-message "No opening of comment!")
            (save-excursion 
                (set-mark)
                (beginning-of-line)
                (if (! (looking-at ".*{"))
                    (progn 
                           (search-reverse "{")
                           (narrow-region)
                           (filter-region "fmt")
                           (beginning-of-file)
                           (while (! (eobp))
                                  (next-line)
                                  (beginning-of-line)
                                  (insert-string "{ ")
                                  (end-of-line))
                           (widen-region)))))))
    
(defun 
    (change-indentation colno
        (set-mark)
        (save-excursion 
            (if (< (mark) (dot))
                (progn 
                       (end-of-line)
                       (exchange-dot-and-mark)
                       (beginning-of-line))
                (progn 
                       (beginning-of-line)
                       (exchange-dot-and-mark)
                       (end-of-line)))
            (narrow-region)
            (end-of-file)
            (beginning-of-line)
            (setq colno (+ (current-indent) (arg 1)))
            (delete-white-space)
            (if (> colno 1)
                (to-col colno))
            (beginning-of-line)
            (while (! (bobp))
                   (previous-line)
                   (setq colno (+ (current-indent) (arg 1)))
                   (delete-white-space)
                   (if (> colno 1)
                       (to-col colno))
                   (beginning-of-line))
            (widen-region))))
    
(defun 
    (dedent-region
        (change-indentation (- 0 indent-change))))
    
(defun 
    (indent-region
        (change-indentation indent-change)))
    
(defun 
    (make-pascal-structure name type colno
        (setq name (get-tty-string "Structure name: "))
        (if (= name "")
            (error-occurred "Aborted."))
        (setq colno (current-column))
        (setq type (get-tty-string "Result (<enter> if procedure): "))
        (newline)
        (newline)
        (to-col colno)
        (if (= type "")
            (insert-string (concat "procedure " name))
            (insert-string (concat "function " name)))
        (save-excursion
            (insert-string ";")
            (newline)
            (newline)(to-col colno)
            (insert-string "var")
            (newline)
            (newline)
            (to-col colno)
            (insert-string (concat "BEGIN  { " name " }"))
            (newline)
            (to-col colno)
            (insert-string (concat "END;   { " name " }"))
            (newline))
        (if (!= type "")
            (save-excursion
                (insert-string (concat ":  " type))))
    ))
    
(defun 
    (indent-pascal colno
        (if (eolp)
                (newline-and-indent)
            (progn
                  (save-excursion 
                      (end-of-line)
                      (if (eobp)
                          (newline)))
                  (setq colno (current-indent))
                  (beginning-of-line)
                  (cond
                       (| (= abbrev-mode 0)
                          (& (looking-at ".*{") (! (looking-at ".*}"))))
                       (progn 
                              (end-of-line)
                              (newline-and-indent)
                              (insert-string "{ ")
                              (setq abbrev-mode 0)
                       )
                       (looking-at "[ \t]*begin\\b")
                       (progn 
                              (save-excursion 
                                  (previous-line)
                                  (beginning-of-line)
                                  (forward-word)
                                  (delete-previous-word)
                                  (yank-from-killbuffer))
                              (end-of-line)
                              (newline-and-indent)
                              (save-excursion 
                                  (newline-and-indent)
                                  (insert-string "end;  { ")
                                  (yank-from-killbuffer)
                                  (insert-string " }")))
                       (looking-at "[ \t]*end\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (- colno indent-change)))
                       (looking-at "[ \t]*var\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at "[ \t]*type\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at "[ \t]*const\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at "[ \t]*if\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at "[ \t]*else\\b")
                       (progn 
                              (save-excursion 
                                  (previous-line)
                                  (beginning-of-line)
                                  (if (looking-at ".*;")
                                      (progn 
                                             (search-forward ";")
                                             (delete-previous-character)
                                             (message
                                                     "Removed semi-colon before 'else'."))))
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at "[ \t]*while\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at "[ \t]*repeat\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change))
                              (save-excursion 
                                  (newline)
                                  (to-col colno)
                                  (insert-string "until ;")))
                       (looking-at "[ \t]*case\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at "[ \t]*for\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at "[ \t]*loop\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at "[ \t]*exit\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (looking-at ".*record\\b")
                       (progn 
                              (beginning-of-line)
                              (forward-word)
                              (delete-previous-word)
                              (yank-fromkkillbuffer)
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change))
                              (save-excursion 
                                  (newline-and-indent)
                                  (insert-string "end;  { ")
                                  (yank-from--buffer)
                                  (insert-string " }")))
                       (looking-at "[ \t]*with\\b")
                       (progn 
                              (end-of-line)
                              (newline)
                              (to-col (+ colno indent-change)))
                       (progn
                             (end-of-line)
                             (newline-and-indent)))))))

(pascal-mode)

;  * pascal mode  - automatic expansion of forms

(defun (dmm_get-word
	(setq &tem (concat &tem &var))
	(setq &var (char-to-string (following-char)))
	(forward-character)
	(if (!= &var " ")
		(dmm_get-word))
	(setq &var "")))

(defun (begin-end-pair
	(message prefix-argument)
	(sit-for 20)
	(if (= prefix-argument 4)
		(message "inside")
		(sit-for 30)
		(backward-word)
		(setq pas_indent (current-column))
		(delete-next-word)
		(insert-string (abbrev-expansion)))
	(message "outside")
	(sit-for 30)
	(setq &dummy "0")	; this allows if-then-elses
	(where-comment-switch)
	(set-up-comment)
	(insert-string (concat "{ " &comment-marker " }"))
	(newline-and-indent)
	(if (= &if-else-dummy "0")
		(insert-string "END;")
		(insert-string "END"))
	(setq &dummy "1")
	(where-comment-switch)
	(insert-string (concat "{ " &comment-marker " }"))
	(beginning-of-line)
	(newline-and-backup)
	(setq pas_indent 
		(+ pas_indent current-indent))
	(to-col pas_indent)
	(novalue)))

 (defun (where-comment-switch
	(if (= &comment-switch "0")
		(to-col &comment-column)
		(if (= &dummy "0")
			(insert-string "  ")
			(insert-string "   ")))))

(defun (set-up-comment
	(if (= &switch-to-arg "1")
		(setq &comment-marker (get-tty-string "Comment: "))
		(= &switch-to-arg "0")
			(setq &comment-marker 
				(concat &comment-starter
				        " " 
					&argument-marker)))
	(setq &switch-to-arg "1")))

(defun
	(decide-which-type-comment
		(if (= "0" comment_mode)
			(setq &switch-to-arg "0"))
		(novalue)))

(defun 
	(for-do-pair temp-marker
		(backward-word)
		(setq pas_indent (current-column))
		(forward-word)
		(setq &comment-starter "for")
		(setq &arg-bool "1")
		(setq &arg-name "var-nam? ")
		(set-up-argument)
		(insert-string (concat " " &argument-marker " := "))
		(setq temp-marker &argument-marker)
		(setq &arg-name "bounds-of-iteration? ")
		(set-up-argument)
		(setq &arg-bool "0")
		(insert-string (concat &argument-marker " do "))
		(setq &argument-marker
			(concat temp-makrer " := " &argument-marker))
		(setq pas_indent
			(+ pas_indent 3))
		(newline)
		(to-col pas_indent)
		(decide-which-type-comment)
		(insert-string "BEGIN")
		(provide-prefix-argument 2 (begin-end-pair))))

(defun 
	(while-do-pair
		(backward-word)
		(setq pas_indent (current-column))
		(delete-next-word)
		(insert-string (abbrev-expansion))
		(setq &comment-starter "while")
		(set-up-argument)
		(insert-string (concat " " &argument-marker " DO"))
		(newline)
		(to-col pas_indent)
		(message "hi there")
		(sit-for 30)
		(decide-which-type-comment)
		(message "inside here")
		(sit-for 20)
		(begin-end-pair 1)
		(novalue)))

(defun 
	(repeat-until-pair
		(backward-word)
		(setq pas_indent (current-column))
		(insert-string (abbrev-expansion))
		(setq &comment-starter "repeat-until ")
		(newline)
		(to-col pas_indent)
		(insert-string (concat " " &argument-marker ";"))
		(to-col &comment-column)
		(set-up-comment)
		(insert-string (concat "{ " &comment-marker "}"))
		(previous-line)
		(to-col &comment-column)
		(insert-string (concat "{ " &comment-marker "}"))
		(newline-and-indent)
		(setq &current-column
			(+ pas_indent current-indent))
		(to-col pas_indent)
		(novalue)))

(defun 
	(if-then-pair
		(backward-word)
		(setq pas_indent (current-column))
		(insert-string (abbrev-expansion))
		(setq &comment-starter "if")
		(set-up-argument)
		(insert-string (concat " " &argument-marker " then "))
		(newline)
		(to-col pas_indent)
		(decide-which-type-comment)
		(provide-prefix-argument 2 (begin-end-pair))
		(novalue)))

(defun 
	(pascal-tab
		(setq pas_indent
			(+ (current-column) current-indent))
		(to-col pas_indent)))

(defun 
	(pascal-detab
		(beginning-of-line)
		(to-col (- (current-indent) current-indent))
		(insert-string ".")
		(delete-white-space)
		(delete-previous-character)
		(end-of-line)))

(defun 
	(pascal-to-comment
		(to-col &comment-column)))

(defun 
	(if-else
		(setq &if-else-dummy 1)
		(if-then-pair)
		(setq &if-else-dummy 0)
		(next-line)
		(end-of-line)
		(newline-and-indent)
		(insert-string "ELSE")
		(newline-and-indent)))

(defun 
	(pascal-return-semi
		(insert-string ";")
		(if (eolp)
			(newline-and-indent))))

(defun 
	(pascal-square
		(insert-string "[]")
		(backward-character)))

(defun
	(pascal-paren
		(insert-string "()")
		(backward-character)))

(defun
	(pascal-quote
		(insert-string """""")
		(backward-character)))	

(defun 
	(set-up-argument
		(if (= &arg-bool "0")
			(setq &argument-marker (get-tty-string "Argument: "))
			(setq &argument-marker (get-tty-string &arg-name)))
		(novalue)))
