; Pascal support.
; Richard Cohn
; 1-Nov-82


; New bindings:
;	), }	parenthesis matcher (from Doug Phillips)
; 	ESC-p	make a procedure or function (from Doug Phillips)
; 	^M	indent pascal (try it out)
; 	^X->	Indent this line by two.
; 	^X-<	Dedent this line by two.
;	current-indent (default 2) determines size of indenting


(defun
    (pascal-mode
	(setq-default indent-change 2)
	(setq mode-string "pascal")
	(use-syntax-table "pascal")
	(setq abbrev-mode 1)
	(local-bind-to-key "indent-pascal" "\^M")
	(local-bind-to-key "newline" "\e\^M")
	(local-bind-to-key "make-pascal-procedure" "\ep")
	(local-bind-to-key "show-matching-paren" ')')
	(local-bind-to-key "leave-and-show" '}')
	(local-bind-to-key "indent-region" "\^X>")
	(local-bind-to-key "dedent-region" "\^X<")
	(local-bind-to-key "enter-comment-mode" '{')
	;	(local-bind-to-key "enter-fuzzy-comment" '*')
	;	(local-bind-to-key "end-fuzzy-comment" ')')
	(local-bind-to-key "toggle-comment-mode" "'")
	(modify-syntax-entry "w    _")
	(modify-syntax-entry "(){ *(")
	(modify-syntax-entry "   })*")
	(modify-syntax-entry ")(   )")
	(modify-syntax-entry "(]   [")
	(modify-syntax-entry ")[   ]")
	(modify-syntax-entry "(}{  {")
	(modify-syntax-entry "){ } }")
	(error-occurred (pascal-mode-hook))
	(novalue)
    ) ;;; end of pascal-mode
    
    ;  Lisp-style cond.
    (cond n running
	  (setq n 1)
	  (setq running 1)
	  (while (& running (< n (nargs)))
		 (if (arg n)
		     (setq running 0)
;		     (setq n (+ n 2)) ))
		     (+= n 2) ))
	  (arg (+ n 1)))
    
    (toggle-comment-mode
	(if (= abbrev-mode 0)
	    (setq abbrev-mode 1)
	    (setq abbrev-mode 0))
	(insert-character (last-key-struck))
    ) ;;; end of toggle-comment-mode
    
    (enter-comment-mode
	(setq abbrev-mode 0)
	(insert-character (last-key-struck))
    ) ;;; end of enter-comment-mode
    
    (leave-comment-mode
	(setq abbrev-mode 1)
	(insert-character (last-key-struck))
    ) ;;; end of leave-comment-mode
    
;    (enter-fuzzy-comment last-key prev-char
;	(setq last-key (last-key-struck))
;	(setq prev-char (preceding-char))
;	(if (& (= last-key  42)		; 42 = Asterisk
;	       (= prev-char 40))	; 40 = open paren
;	    (progn
;		(setq abbrev-mode 0)
;		(insert-string "*"))
;	    (insert-character last-key)))
    
;    (end-fuzzy-comment last-key prev-char
;	(setq last-key (last-key-struck))
;	(setq prev-char (preceding-char))
;	(if (& (= last-key  41)		; 41 = close paren
;	       (= prev-char 42))	; 42 = Asterisk
;	    (setq abbrev-mode 1))
;	(insert-character last-key))
    
    (show-matching-paren
	(insert-character (last-key-struck))
	(save-excursion 
	    (backward-paren)
	    (if (dot-is-visible)
		(sit-for 5)
		(progn 
		       (beginning-of-line)
		       (set-mark)
		       (end-of-line)
		       (message (region-to-string))))))
    
    (leave-and-show
	(setq abbrev-mode 1)
	(show-matching-paren))
    
    (new-leave-and-show
	(setq abbrev-mode 1)
	(if (error-occurred (show-matching-paren))
	    (error-message "No opening of comment!")
	    (save-excursion 
		(set-mark)
		(beginning-of-line)
		(if (! (looking-at ".*{"))
		    (progn 
			   (search-reverse "{")
			   (narrow-region)
			   (fast-filter-region "fmt")
			   (beginning-of-file)
			   (while (! (eobp))
				  (next-line)
				  (beginning-of-line)
				  (insert-string "{ ")
				  (end-of-line))
			   (widen-region))))))
    
    (change-indentation colno	; indent by arg for region
	(save-excursion 
	    (if (< (mark) (dot))
		(progn 
		       (end-of-line)
		       (exchange-dot-and-mark)
		       (beginning-of-line))
		(progn 
		       (beginning-of-line)
		       (exchange-dot-and-mark)
		       (end-of-line)))
	    (narrow-region)
	    (end-of-file)
	    (beginning-of-line)
	    (setq colno (+ (current-indent) (arg 1)))
	    (delete-white-space)
	    (if (> colno 1)
		(to-col colno))
	    (beginning-of-line)
	    (while (! (bobp))
		   (previous-line)
		   (setq colno (+ (current-indent) (arg 1)))
		   (delete-white-space)
		   (if (> colno 1)
		       (to-col colno))
		   (beginning-of-line))
	    (widen-region)))
    
    (dedent-region 		; dedents region
	(change-indentation (- 0 indent-change))) 
    
    (indent-region colno	; indents region
	(change-indentation indent-change)) 
    
    (make-pascal-procedure name type colno
	(setq name (get-tty-string "Procedure name: "))
	(if (= name "")
	    (error-occurred "Aborted."))
	(setq colno (current-column))
	(setq type (get-tty-string "Result: "))
	(newline)
	(newline)
	(insert-string "{********************************")
	(insert-string "********************************}")
	(newline)
	(newline)(to-col colno)
	(if (= type "")
	    (insert-string (concat "procedure " name))
	    (insert-string (concat "function " name)))
	(save-excursion
	    (insert-string ";")
	    (newline)
	    (newline)(to-col colno)
	    (insert-string "var")
	    (newline)
	    (newline)(to-col colno)
	    (insert-string (concat "begin  { " name " }"))
	    (newline)(to-col colno)
	    (insert-string (concat "end;  { " name " }"))
	    (newline))
	(if (!= type "")
	    (save-excursion
		(insert-string (concat ":  " type))))
    );;; end of make-pascal-procedure
    
    (indent-pascal colno
	(if (! (eolp))
	    (save-excursion 
		(newline-and-indent))
	    (progn
		  (save-excursion 
		      (end-of-line)
		      (if (eobp)
			  (newline)))
		  (setq colno (current-indent))
		  (beginning-of-line)
		  (cond
		       (| (= abbrev-mode 0)
			  (& (looking-at ".*{") (! (looking-at ".*}"))))
		       (progn 
			      (end-of-line)
			      (newline-and-indent)
			      (insert-string "{ ")
			      (setq abbrev-mode 0)
		       )
		       (looking-at "[ \t]*begin\\b")
		       (progn 
			      (save-excursion 
				  (previous-line)
				  (beginning-of-line)
				  (forward-word)
				  (kill-previous-word)
				  (yank-kill))
			      (end-of-line)
			      (newline-and-indent)
			      (save-excursion 
				  (newline-and-indent)
				  (insert-string "end;  { ")
				  (yank-kill)
				  (insert-string " }")))
		       (looking-at "[ \t]*end\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (- colno indent-change)))
		       (looking-at "[ \t]*var\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at "[ \t]*type\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at "[ \t]*const\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at "[ \t]*if\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at "[ \t]*else\\b")
		       (progn 
			      (save-excursion 
				  (previous-line)
				  (beginning-of-line)
				  (if (looking-at ".*;")
				      (progn 
					     (search-forward ";")
					     (delete-previous-character)
					     (message
						     "Removed semi-colon before 'else'."))))
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at "[ \t]*while\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at "[ \t]*repeat\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change))
			      (save-excursion 
				  (newline)
				  (to-col colno)
				  (insert-string "until ;")))
		       (looking-at "[ \t]*case\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at "[ \t]*for\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at "[ \t]*loop\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at "[ \t]*exit\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       (looking-at ".*record\\b")
		       (progn 
			      (beginning-of-line)
			      (forward-word)
			      (kill-previous-word)
			      (yank-kill)
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change))
			      (save-excursion 
				  (newline-and-indent)
				  (insert-string "end;  { ")
				  (yank-kill)
				  (insert-string " }")))
		       (looking-at "[ \t]*with\\b")
		       (progn 
			      (end-of-line)
			      (newline)
			      (to-col (+ colno indent-change)))
		       1
		       (progn
			     (end-of-line)
			     (newline-and-indent))))))
    
) ;;; end of massive Defun

(novalue)
