;---------------------------------------------------------------------------
; scribe-bib.ml
;
;SYNOPSIS
;
; Emacs Scribe bibliography entry package
; 
;KEYBINDINGS
;
;DESCRIPTION
; 
; ^g can now be used to abort out of an entry, with the closing paren added
; : does a recursive edit
; @ as the first character of a value doesn't quote the value
; \@ quotes the @
; The following functions expedite the entry of the various bibliography
; types supported by Scribe.
;
;HISTORY
; John A. Nestor March 1982 created.
; Ed Frank added lots of useful functions
;----------------------------------------------------------------------




(defun 
    (get-keypair field	 continue	; construct an "attribute=value" line
	(setq continue 1)
	(while continue
		 (setq field (get-tty-string  (concat (arg 1 "bogus1") ":")))
		 (if (= (substr field 1 1) ":")
		     (recursive-edit)
		     (= field "?")
		     (progn
			 (message  ": for recursive edit, @<value> to entry a scribe string. \@ to quote the @")
			 (sit-for 1))
		     (setq continue 0)))
	(if (= (substr field 1 1) "@")
	    (setq field (substr field 2 (- (length field) 1)))
	    (progn
		(if (= (substr field 1 1) "\\")
		    (setq field (substr field 2 (- (length field) 1))))
		(setq field (concat """" field """"))))
	(if (& (!= field "") (!= field """"""))
	    (insert-string
		(concat ",\n\t" (arg 1 "bogus2") "=" field)))))

(defun 				; generate an @article bib entry
  (@article
    (newline)
    (newline)
    (insert-string "@Article(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "Journal")
      (get-keypair "Volume")
      (get-keypair "Number")
      (get-keypair "Pages")
      (get-keypair "Month")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  
  (@book
    (newline)
    (newline)
    (insert-string "@Book(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "Publisher")
      (get-keypair "Address")
      (get-keypair "Series")
      (get-keypair "Volume")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  
  (@booklet
    (newline)
    (newline)
    (insert-string "@Book(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "HowPublished")
      (get-keypair "Address")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  
  (@inbook
    (newline)
    (newline)
    (insert-string "@InBook(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "Chapter")
      (get-keypair "Pages")
      (get-keypair "Publisher")
      (get-keypair "Address")
      (get-keypair "Series")
      (get-keypair "Volume")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  
  (@incollection
    (newline)
    (newline)
    (insert-string "@InCollection(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "BookTitle")
      (get-keypair "Chapter")
      (get-keypair "Pages")
      (get-keypair "Editor")
      (get-keypair "Publisher")
      (get-keypair "Address")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  
  (@inproceedings
    (newline)
    (newline)
    (insert-string "@InProceedings(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "Organization")
      (get-keypair "BookTitle")
      (get-keypair "Editor")
      (get-keypair "Address")
      (get-keypair "Pages")
      (get-keypair "Month")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  (@manual
    (newline)
    (newline)
    (insert-string "@Manual(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "Edition")
      (get-keypair "Organization")
      (get-keypair "Address")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  (@mastersthesis
    (newline)
    (newline)
    (insert-string "@MastersThesis(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "School")
      (get-keypair "Month")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  
  (@misc
    (newline)
    (newline)
    (insert-string "@Misc(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "HowPublished")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  
  (@phdthesis
    (newline)
    (newline)
    (insert-string "@PhDThesis(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "School")
      (get-keypair "Month")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  (@proceedings
    (newline)
    (newline)
    (insert-string "@Proceedings(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Organization")
      (get-keypair "Editor")
      (get-keypair "Publisher")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "Address")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  
  (@techreport
    (newline)
    (newline)
    (insert-string "@TechReport(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "Institution")
      (get-keypair "Number")
      (get-keypair "Type")
      (get-keypair "Month")
      (get-keypair "Year")
      (get-keypair "Note"))
    (insert-string ")")
    (newline))
  
  
  (@unpublished
    (newline)
    (newline)
    (insert-string "@Unpublished(")
    (error-occurred
      (insert-string (arg 1 "CodeWord: "))
      (get-keypair "Author")
      (get-keypair "Key")
      (get-keypair "Title")
      (get-keypair "Note"))
    (insert-string ")")
    (newline)))
(novalue)
