; scribe.ml
;
; simple scribe-mode help
;
(declare-buffer-specific scribe-bib-name)

(setq-default &last-scribe-environment "Itemize")

(defun (scribe-fill
	   (save-excursion
	       (save-restriction
		   (set-mark)
		   (beginning-of-line)
		   (narrow-region)
		   (error-occurred (provide-prefix-argument right-margin
				      (forward-character)))
		   (while (= (preceding-char) ' ')
			  (backward-character))
		   (while (& (! (bolp)) (> (current-column) right-margin))
			  (backward-character)
			  (while (& (! (bolp)) (!= (following-char) ' '))
				 (backward-character))
			  (while (= (preceding-char) ' ')
				 (backward-character)))
		   (if (! (bolp))
		       (progn (delete-white-space)
			      (insert-character '\n')
			      (to-col left-margin)
			      (insert-string prefix-string)
			      (if (& (bolp) (= (following-char) '@'))
				  (insert-character ' '))))
	       ))
	   1))


(defun
  (scribe-file-name filename
    (save-excursion
      (setq filename (current-file-name))
      (temp-use-buffer "scribe-file-name-work-buffer")
      (erase-buffer)
      (insert-string filename)
      (end-of-file)
      (if  (error-occurred
	     (error-occurred (search-reverse "."))
	     (set-mark)
	     (search-reverse "/")
	     (forward-character))
	   ""
	   (region-to-string))))

  (get-scribe-bib-name  file-name default-file-name mss-buffer-name
    bib-file-name
    (load "scribe-bib.ml")
    (save-window-excursion
      (setq mss-buffer-name (current-buffer-name))
      (setq default-file-name
	(concat (scribe-file-name) ".bib"))
      (setq file-name
	(get-tty-string
	  (concat "use bibliography file "
	    "[" default-file-name "]? ")))
      (if (= file-name "")
	  (setq file-name default-file-name))
      (error-occurred (visit-file file-name))
      (scribe-mode)
      (setq case-fold-search 1)
      (setq bib-file-name (current-file-name))
      (temp-use-buffer mss-buffer-name)
      (setq scribe-bib-name bib-file-name)))

  (cite-do-it citation answer continue
    (save-window-excursion
      (if (= scribe-bib-name "")
	  (get-scribe-bib-name))
      (error-occurred (visit-file  scribe-bib-name))
      (beginning-of-file)
      (setq citation (arg 1 "Citation? "))
      (setq continue 1)
        (while
	    (& continue
	       (error-occurred (re-search-forward (concat "[[{<(\"'][ \t]*"
							 citation))))
	       (setq answer
		     (get-tty-string
			 (concat
			     "No entry for " citation ". ^G - abort, @<bib-type> - new entry, or new citation? ")))
	       (if (= (substr answer 1 1) "@")
		   (progn
		       (end-of-file)
		       (execute-mlisp-line (concat "(" answer " """ citation """)"))
		       (setq continue 0))
		   (setq citation answer))))
    citation)

  (cite
    (insert-string
      (concat " @cite[" (cite-do-it (get-tty-string "Citation? ")) "] "))
    (novalue))
  (cite-check word
      (forward-character)
      (backward-word)
      (set-mark)
      (forward-word)
      (setq word (region-to-string))
      (setq word (cite-do-it word))
      (erase-region)
      (insert-string word)
      (novalue)))

(defun
    (apply-look go-forward
	(save-excursion c
	    (if (! (eolp)) (forward-character))
	    (setq go-forward -1)
	    (backward-word)
	    (setq c (get-tty-character))
	    (if (> c ' ')
		(progn (if (bolp) (insert-character ' '))
		       (insert-character '@')
		       (insert-character c)
		       (insert-character '[')
		       (forward-word)
		       (setq go-forward (dot))
		       (insert-character ']')
		)
	    )
	)
	(if (= go-forward (dot)) (forward-character))
    )
;;
;; &scribe-command: prompt for command name (e.g., "itemize")
;;		and create @begin() ... @end() brackets,
;;		leaving dot between them.
    (&scribe-command sc-cmd
	    (setq sc-cmd (get-tty-string
			     (concat "Scribe command["
			     &last-scribe-environment "]: ")))
	    (if (!= sc-cmd "") (setq &last-scribe-environment sc-cmd))
	    (if (! (bolp )) (newline))
	    (insert-string
	    	"@Begin(" &last-scribe-environment
		")\n\n@End(" &last-scribe-environment ")\n")
	    (provide-prefix-argument 2 (previous-line))
	    (novalue)
    )
;;
;; index-entry: make Scribe index-entry for "current" word
;;		(or n words with prefix-argument)
;;		(or n previous words with negative prefix-argument)
    (index-entry
    	(save-excursion index-word
	    (error-occurred (forward-character))
	    (if (< prefix-argument 0)
		(provide-prefix-argument
		    (- 0 prefix-argument)
		    (backward-word)
		)
		(backward-word)
	    )
	    (set-mark)
	    (if (< prefix-argument 0)
	    	(provide-prefix-argument
		    (- 0 prefix-argument)
		    (forward-word)
		)
		(prefix-argument-loop (forward-word))
	    )
	    (setq index-word (region-to-string))
	    (end-of-line)
	    (insert-string "\n@Index[" index-word "]")
	    (novalue)
	)
    )
    (&scribe-paren-pause dot instabs
	(setq instabs (bolp))
	(setq dot (dot))
	(insert-character (last-key-struck))
	(save-excursion
	    (backward-paren)
	    (if (dot-is-visible)
		(sit-for 5))))
)

(defun
    (scribe-mode
	(remove-all-local-bindings)
	(if (& (! buffer-is-modified) (! read-only))
	    (save-excursion
		(error-occurred
		    (goto-character 2000)
		    (search-reverse "LastEditDate=""")
		    (search-forward """")
		    (set-mark)
		    (search-forward """")
		    (backward-character)
		    (erase-region)
		    (insert-string (current-time))
		    (setq buffer-is-modified 0)
		)
	    )
	)
	(use-abbrev-table "text-mode")
	(local-bind-to-key "cite" "\ec")
	(local-bind-to-key "cite-check" "\eC")
	(local-bind-to-key "justify-paragraph" "\ej")
	(local-bind-to-key "apply-look" "\el")
	(local-bind-to-key "apply-look" "\^X\^L")
	(local-bind-to-key "index-entry" "\ei")
	(local-bind-to-key "&scribe-command" "\es")
	(local-bind-to-key "&scribe-paren-pause" ')')
	(local-bind-to-key "&scribe-paren-pause" ']')
	(local-bind-to-key "&scribe-paren-pause" '}')
	(set-auto-fill-hook "scribe-fill")
	(setq scribe-bib-name "")
	(setq right-margin 77)
	(setq mode-string "Scribe")
	(setq case-fold-search 1)
	(use-syntax-table "text-mode")
	(modify-syntax-entry "w    -'")
	(modify-syntax-entry "()   (")
	(modify-syntax-entry ")(   )")
	(modify-syntax-entry "(]   [")
	(modify-syntax-entry ")[   ]")
	(modify-syntax-entry "(}   {")
	(modify-syntax-entry "){   }")
	(modify-syntax-entry """    """)
	(setq left-margin 1)
	(error-occurred (scribe-mode-hook))
	(novalue)
    )
)
(novalue)
