;----------------------------------------------------------------------------
; seiko.ml -- seiko 8620 terminal support package
;
;SYNOPSIS
;   function-key	-- returns key-sequence value of nth function key
;
;KEY BINDINGS
;   See below.
;
;DESCRIPTION
;   This package, when loaded, sets up nice global bindings for the special
;keys on a Seiko 8620 terminal. It also provides an easy way of generating
;the values of the remaining (numbered) function keys for use in the remainder
;of the user's profile.
;
;NOTE
;   In case your site lacks one, here's the Seiko termcap description we use:
;
;s1|seiko:\
;	:co#80:li#24:am:cl=\E[;H\E[2J:bs:cm=\E[%i%2;%2H:nd=\E[C:up=\E[A:\
;	:ce=\E[K:cd=\E[J:so=\E[7m:se=\E[m:us=\E[4m:ue=\E[m:\
;	:is=\E>\E[3l\E[4l\E[5l\E[7h\E[8h:ks=\E[1h\E=:ke=\E[1l\E>:\
;	:if=/usr/lib/tabset/vt100:ku=\E[A:kd=\E[B:kr=\E[C:kl=\E[D:\
;	:kh=\E[H:k1=\E[P:k2=\E[Q:k3=\E[R:k4=\E[S:pt:sr=\EM:xn:\
;	:dl=\E[M:al=\E[L:
;
;HISTORY
;   05-Jul-84  Eric S. Raymond and Tom Donohue at Rabbit Software
;	Created.
;----------------------------------------------------------------------------

; Named motion and editing keys
(bind-to-key "delete-previous-character"	"\^h")	; BACKSPACE
(bind-to-key "newline-and-backup"		"\e[L")	; INSERT LINE
(bind-to-key "delete-to-killbuffer"		"\e[0K"); LINE ERASE
(bind-to-key "delete-next-character"		"\e[P")	; DELETE CHAR
(bind-to-key "backward-character"		"\e[D")	; <
(bind-to-key "previous-line"			"\e[A")	; ^
(bind-to-key "next-line"			"\e[B")	; v
(bind-to-key "forward-character"		"\e[C")	; >
(bind-to-key "beginning-of-file"		"\^^")	; HOME

;----------------------------------------------------------------------------
; Give the following functions a single numeric argument each.
; They will return the keystroke value of the corresponding numbered key.
;
; Use these with (bind-to-key) and (local-bind-to-key) as in
;    (bind-to-key "next-paragraph" (function-key 10))
;----------------------------------------------------------------------------

(defun
	(function-key
		(concat "\^A" (char-to-string (+ (arg 1) 64)) "\r")
	)
	(shifted-function-key
		(concat "\^A" (char-to-string (+ (arg 1) 96)) "\r")
	)
)

; seiko.ml ends here --------------------------------------------------------
