;-----------------------------------------------------------------------------
; vt100.ml 		vt100 terminal support package
;
;SYNOPSIS
;   function-key	-- returns key-sequence value of nth function key
;
;KEY BINDINGS
;   See below.
;
;DESCRIPTION
;   This package, when loaded, sets up nice global bindings for the special
;keys on a VT100 terminal or lookalike. It also provides an easy way to
;generate the values of the remaining (numbered) function keys for use in the
;remainder of the user's profile.
;
;NOTE
;   In case your site lacks one here's the VT100 termcap description we use:
;
;d1|vt100|vt-100|pt100|pt-100|dec vt100:\
;	:co#80:li#24:am:cl=50\E[;H\E[2J:bs:cm=5\E[%i%2;%2H:nd=2\E[C:up=2\E[A:\
;	:ce=3\E[K:cd=50\E[J:so=2\E[7m:se=2\E[m:us=2\E[4m:ue=2\E[m:\
;	:is=\E>\E[?3l\E[?4l\E[?5l\E[?7h\E[?8h:ks=\E[?1h\E=:ke=\E[?1l\E>:\
;	:if=/usr/lib/tabset/vt100:ku=\EOA:kd=\EOB:kr=\EOC:kl=\EOD:\
;	:kh=\E[H:k1=\EOP:k2=\EOQ:k3=\EOR:k4=\EOS:pt:sr=5\EM:xn:
;
;HISTORY
;  {3}	22-Nov-84  Mike Gallaher at unipress
;	Added standard keypad and labeled key definitions.
;  {2}	07-Aug-84  Eric S. Raymond
;	Fixed a documentation bug.
;  {1}	12-Jul-84  Eric S. Raymond and Dave Langdon at Rabbit Software
;	Created.
;-----------------------------------------------------------------------------

; Named motion and editing keys
(bind-to-key "delete-previous-character"	"\^h")	; BACKSPACE
(bind-to-key "delete-next-character"		"\e[P")	; DELETE
(bind-to-key "backward-character"		"\e[D")	; <
(bind-to-key "previous-line"			"\e[A")	; ^
(bind-to-key "next-line"			"\e[B")	; v
(bind-to-key "forward-character"		"\e[C")	; >
; 
; set up standard keypad bindings	{3}
; 
(send-string-to-terminal 	  "\e=")	; set keypad application mode
(bind-to-key "previous-page"      "\eOw")	; keypad 7
(bind-to-key "previous-line"      "\eOx")	; keypad 8
(bind-to-key "next-page"          "\eOy")	; keypad 9
(bind-to-key "backward-character" "\eOt")	; keypad 4
(bind-to-key "argument-prefix"    "\eOu")	; keypad 5
(bind-to-key "forward-character"  "\eOv")	; keypad 6
(bind-to-key "backward-word" 	  "\eOq")	; keypad 1
(bind-to-key "next-line"	  "\eOr")	; keypad 2
(bind-to-key "forward-word" 	  "\eOs")	; keypad 3
(bind-to-key "expand-mlisp-word" "\eOp")	; keypad 0
(bind-to-key "nothing" "\eOn")			; keypad .
(bind-to-key "nothing" "\eOm")			; keypad -
(bind-to-key "nothing" "\eOl")			; keypad ,
; 
; Bind functions that make sense to the labeled special keys
; 
(bind-to-key "newline-and-backup" "\e[L")	; ins line
(bind-to-key "delete-next-character" "\e[P")	; del char
(bind-to-key "kill-to-end-of-line" "\e[M")	; del line
(bind-to-key "previous-page" "\e[T")		; prev scrn
(bind-to-key "next-page" "\e[S")		; next scrn
(bind-to-key "beginning-of-window" "\e[f")	; home
(bind-to-key "newline" "\eOM")			; enter

;-----------------------------------------------------------------------------
; Give the following function a single numeric argument.
; It will return the keystroke value of the corresponding numbered PF-key.
;
; Use with (bind-to-key) and (local-bind-to-key) as in
;    (bind-to-key "next-paragraph" (function-key 10))
;-----------------------------------------------------------------------------

(defun
	(function-key
		(concat "\eO" (char-to-string (+ (arg 1) 'O')))
	)
)

; vt100.ml ends here -----------------------------------------------------
